"""Performs speedtest-cli tests and pushes results to Prometheus Pushgateway."""

import argparse
import socket
import threading

import prometheus_client
import speedtest

PARSER = argparse.ArgumentParser(
    description='Export speedtest metrics to Prometheus Pushgateway.')
PARSER.add_argument(
    '-s', '--source_address', metavar='addr', type=str,
    help='IP address for speedtest to bind to.')
PARSER.add_argument(
    '-t', '--timeout', metavar='sec', default=30, type=int,
    help='Speedtest timeout, seconds.')
PARSER.add_argument(
    '-p', '--pushgateway', metavar='host:port', default='localhost:9091',
    type=str, help='Address of Prometheus Pushgateway.')
PARSER.add_argument(
    '-n', '--name', metavar='name', default='prometheus_speedtest', type=str,
    help='Job name to report Prometheus metrics as.')


class Error(Exception):
  """Base class for exceptions raised in this module."""


class TimeoutError(Exception):
  """Speedtest timeout."""


class PrometheusSpeedtest(object):
  """Enapsulates behavior performing and reporting results of speedtests."""

  def __init__(self, pushgateway, jobname, source_address=None, timeout=None,
               *args, **kwargs):
    """Instantiates a PrometheusSpeedtest object.

    Args:
      pushgateway: str - host:port of Prometheus Pushgateway.
      jobname: str - jobname to report to Prometheus Pushgateway.
      source_address: str - optional network address to bind to.
        e.g. 192.168.1.1.
      timeout: int - optional timeout for speedtest in seconds.
    """
    super(PrometheusSpeedtest, self).__init__(*args, **kwargs)
    self.pushgateway = pushgateway
    self.jobname = jobname
    self.source_address = source_address
    self.timeout = timeout

  def _test(self):
    """Performs speedtest, returns results.

    Returns:
      speedtest.SpeedtestResults object.
    Raises:
      TimeoutError: Speedtest timeout was reached.
    """
    client = speedtest.Speedtest(source_address=self.source_address)

    def perform_test(client):
      """Performs speedtest."""
      client.get_best_server()
      client.download()
      client.upload()

    thread = threading.Thread(target=perform_test, args=(client,))
    thread.start()
    thread.join(self.timeout)
    if thread.is_alive():
      thread.exit()
      raise TimeoutError('Speedtest timeout')
    return client.results

  def _push(self, results):
    """Pushes metrics about a speedtest to the Prometheus Pushgateway.

    Args:
      results: speedtest.SpeedtestResults object.
    """
    registry = prometheus_client.CollectorRegistry()
    download_bps = prometheus_client.Gauge(
        'download_speed_bps', 'Download speed (bit/s)',
        registry=registry)
    download_bps.set(results.download)
    upload_bps = prometheus_client.Gauge(
        'upload_speed_bps', 'Upload speed (bit/s)', registry=registry)
    upload_bps.set(results.upload)
    ping = prometheus_client.Gauge(
        'ping_ms', 'Latency (ms)', registry=registry)
    ping.set(results.ping)

    grouping_key = {
        'instance': socket.gethostname(),
    }

    prometheus_client.push_to_gateway(
        self.pushgateway, job=self.jobname, grouping_key=grouping_key,
        registry=registry)

  def report(self):
    """Performs a speedtest and pushes metrics to Prometheus Pushgateway."""
    results = self._test()
    self._push(results)


def main():
  """Entry point for prometheus_speedtest.py."""
  flags = PARSER.parse_args()
  tester = PrometheusSpeedtest(
      flags.pushgateway, flags.name, flags.source_address, flags.timeout)
  tester.report()


if __name__ == '__main__':
  main()



import os

from setuptools import find_packages
from setuptools import setup

def read_file(rel_path):
  here = os.path.abspath(os.path.dirname(__file__))
  with open(os.path.join(here, rel_path), 'r') as f:
    return f.read()

setup(
  name = 'prometheus_speedtest',
  author = 'Jean-Ralph Aviles',
  author_email = 'jeanralph.aviles+pypi@gmail.com',
  classifiers = [
    'Development Status :: 5 - Production/Stable',
    'Environment :: Console',
    'Intended Audience :: Developers',
    'License :: OSI Approved :: Apache Software License',
    'Operating System :: OS Independent',
    'Topic :: System :: Monitoring',
    'Topic :: System :: Networking :: Monitoring',
    'Programming Language :: Python :: 2',
    'Programming Language :: Python :: 2.4',
    'Programming Language :: Python :: 2.5',
    'Programming Language :: Python :: 2.6',
    'Programming Language :: Python :: 2.7',
    'Programming Language :: Python :: 3',
    'Programming Language :: Python :: 3.1',
    'Programming Language :: Python :: 3.2',
    'Programming Language :: Python :: 3.3',
    'Programming Language :: Python :: 3.4',
    'Programming Language :: Python :: 3.5',
    'Programming Language :: Python :: 3.6',
    'Programming Language :: Python :: 3.7',
    'Programming Language :: Python',
  ],
  description = (
    'Performs speedtest-cli tests and pushes metrics to Prometheus Pushgateway'
  ),
  entry_points = {
    'console_scripts': [
      'prometheus_speedtest=prometheus_speedtest:main',
    ],
  },
  install_requires = [
    'mock>=2.0.0',
    'prometheus_client>=0.2.0',
    'speedtest-cli>=2.0.0',
  ],
  keywords = ['prometheus', 'monitoring', 'speedtest', 'speedtest.net'],
  license='Apache License, Version 2.0',
  long_description = read_file('README.md'),
  long_description_content_type = 'text/markdown',
  py_modules = ['prometheus_speedtest'],
  setup_requires=[
    'setuptools>=38.6.0'
  ],
  url = 'https://github.com/jeanralphaviles/prometheus_speedtest',
  version = read_file('version.txt'),
)
